#include "../include/pool.h"
#include "../include/error.h"
#include "../include/mptp.h"
#include "../include/util.h"

#include <ini.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

lm_pool_t *lm_pool_new(char *name, char *url) {
  lm_pool_t *pool = malloc(sizeof(lm_pool_t));
  pool->available = true;
  pool->name      = name;

  if (!lm_url_init(&pool->url, url)) {
    free(pool);
    return NULL;
  }

  if (!eq(pool->url.protocol, "mptp")) {
    lm_error_set(LM_ERR_PoolNoSupport);
    lm_pool_free(pool);
    return NULL;
  }

  return pool;
}

void lm_pool_test(lm_pool_t *pool) {
  lm_mptp_t packet;
  lm_mptp_init(&packet, true, MPTP_C2S_PING, true);
  lm_mptp_set_host(&packet, pool->url.host);

  int sock = lm_mptp_client_connect(pool->url.host, pool->url.port);
  if (sock == -1) {
    pool->available = false;
    return;
  }

  if (!lm_mptp_client_send(sock, &packet)) {
    pool->available = false;
    goto end;
  }

  if (!lm_mptp_client_recv(sock, &packet)) {
    pool->available = false;
    goto end;
  }

  if (!lm_mptp_client_verify(&packet)) {
    pool->available = false;
    goto end;
  }

  pool->available = MPTP_FLAGS_CODE(&packet) == MPTP_S2C_PONG;
end:
  lm_mptp_close(sock);
  return;
}

int lm_pool_info_handler(void *data, const char *_section, const char *_key, const char *_value) {
  char      *section = (char *)_section, *value = (char *)_value, *key = (char *)_key;
  lm_pool_t *pool = data;

  if (!eq(pool->name, section))
    return 0;

  if (eq(key, "size"))
    pool->info.size = atol(value);
  else if (eq(key, "maintainer"))
    pool->info.maintainer = strdup(value);
  else if (eq(key, "pubkey"))
    pool->info.pubkey = strdup(value);
  else
    return 0;

  return 1;
}

bool lm_pool_info_file(lm_pool_t *pool, char *file) {
  if (ini_parse(file, lm_pool_info_handler, pool) < 0) {
    lm_error_set(LM_ERR_PoolInfoBad);
    return false;
  }
  return true;
}

bool lm_pool_info(lm_pool_t *pool, char *info) {
  if (ini_parse_string(info, lm_pool_info_handler, pool) < 0) {
    lm_error_set(LM_ERR_PoolInfoBad);
    return false;
  }
  return true;
}

void lm_pool_free(lm_pool_t *pool) {
  lm_url_free(&pool->url);
  free(pool);
}
